require 'xml/dom2/domexception'
require 'xml/dom2/domimplementation'
require 'xml/dom2/documentfragment'
require 'xml/dom2/document'
require 'xml/dom2/node'
require 'xml/dom2/nodelist'
require 'xml/dom2/namednodemap'
require 'xml/dom2/characterdata'
require 'xml/dom2/attr'
require 'xml/dom2/element'
require 'xml/dom2/text'
require 'xml/dom2/comment'

require 'xml/dom2/cdatasection'
require 'xml/dom2/documenttype'
require 'xml/dom2/notation'
require 'xml/dom2/entity'
require 'xml/dom2/entityreference'
require 'xml/dom2/processinginstruction'
